@@ -10,7 +10,7 @@ import pandas as pd
 from ipyevents import Event
 from ipywidgets import Widget
 
-__version__ = '0.6.7'
+__version__ = '0.6.8'
 
 class SAMGUI(object):
 
@@ -34,9 +34,13 @@ class SAMGUI(object):
             tab.set_title(0,'LOAD_SAM')
             self.create_plot(0,'SAM not loaded')
             items = [self.stab,tab]
+
+        self.current_tab = 0
+        self.current_sam = sam
         self.SamPlot = widgets.HBox(items)
 
     def init_from_sam(self,sam):
+        self.current_sam=sam
         tab = widgets.Tab()
         self.load_vars_from_sam(sam)
         self.pp_box = self.init_preprocess()
@@ -65,17 +69,23 @@ class SAMGUI(object):
             del self.selected[I]
             del self.selected_cells[I]
             del self.active_labels[I]
+            del self.dd_opts[I]
             del self.marker_genes[I]
             del self.marker_genes_tt[I]
+            del self.gene_expressions[I]
         self.stab.children = (self.stab.children[0],)
 
 
+    def get_scatter(self,i):
+        return self.stab.children[i].data[0]
+
     def load_vars_from_sam(self,sam):
         self.sams = [sam]
         self.GENE_KEY=''
         self.selected = [np.zeros(sam.adata.shape[0],dtype='bool')]
         self.selected[0][:] = True
         self.active_labels = [np.zeros(self.selected[0].size,dtype='int')]
+        self.dd_opts=[['']]
         try:
             self.marker_genes = [np.array(list(sam.adata.var_names))[np.argsort(-sam.adata.var['weights'].get_values())]]
             self.marker_genes_tt = ['Genes ranked by SAM weights.']
@@ -85,6 +95,7 @@ class SAMGUI(object):
 
         self.selected_cells = [np.array(list(sam.adata.obs_names))]
         self.ds = [0]
+        self.gene_expressions = [np.zeros(sam.adata.shape[0])]
 
         self.preprocess_args = sam.preprocess_args.copy()
         self.preprocess_args_init = self.preprocess_args.copy()
@@ -124,6 +135,7 @@ class SAMGUI(object):
                 width = 600, height=600,
                 xaxis_ticks='',
                 xaxis_showticklabels=False,
+                title='',
                 #xaxis_showgrid=False,
                 #xaxis_zeroline=False,
                 #yaxis_showgrid=False,
@@ -136,7 +148,7 @@ class SAMGUI(object):
             f1.update_yaxes(autorange=True)
             f1.update_xaxes(autorange=True)
 
-            f1.data[0].text = list(self.active_labels[-1])
+            f1.data[0].text = list(self.active_labels[i])
             f1.data[0].hoverinfo = 'text'
             f1.set_trait('_config',{'displayModeBar':True, 'scrollZoom':True,'displaylogo':False,'edits':{'titleText':False}})
 
@@ -149,6 +161,9 @@ class SAMGUI(object):
                 self.ds[i] = d
                 d.on_dom_event(self.handle_events)
 
+            slider = self.cs_box.children[12].children[1]
+            slider.set_trait('min',slider.value)
+            slider.set_trait('max',slider.value)
         else:
             f1 = go.FigureWidget()
             f1.add_scattergl(x=[], y=[]);
@@ -178,9 +193,9 @@ class SAMGUI(object):
                     x=0
                 self.cs_box.children[11].children[1].set_trait('value',x)
 
-            elif key == 'shift':
+            elif key == 'Shift':
                 self.irm_genes(None)
-            elif key == 'enter':
+            elif key == 'Enter':
                 self.ism_genes(None)
             elif key == 'x':
                 self.unselect_all(None);
@@ -210,7 +225,9 @@ class SAMGUI(object):
             del self.sams[I]
             del self.selected[I]
             del self.selected_cells[I]
+            del self.gene_expressions[I]
             del self.active_labels[I]
+            del self.dd_opts[I]
             del self.marker_genes[I]
             del self.marker_genes_tt[I]
 
@@ -219,6 +236,7 @@ class SAMGUI(object):
 
     """ BEGIN PREPROCESS INIT"""
     def init_preprocess(self):
+        """
         pdata = widgets.Button(
             description = 'Process data',
             disabled = False,
@@ -226,6 +244,7 @@ class SAMGUI(object):
             icon = ''
         )
         pdata.on_click(self.preprocess_sam)
+        """
         fgenes = widgets.Checkbox(
             value=self.preprocess_args.get('filter_genes',True),
             description='Filter genes'
@@ -328,7 +347,7 @@ class SAMGUI(object):
         )
 
 
-        pp = widgets.VBox([pdata,
+        pp = widgets.VBox([#pdata,
                            widgets.HBox([dfts,fgenes]),
                            norm,
                            sumnorm,
@@ -379,6 +398,9 @@ class SAMGUI(object):
             path = self.pp_box.children[6].children[1].value
 
         filetype = path.split('.')[-1]
+        if filetype == 'gz':
+            filetype = path.split('.')[-2]
+
         sam=SAM()
         if filetype == 'h5ad' or filetype == 'csv':
             sam.load_data(path)
@@ -397,7 +419,7 @@ class SAMGUI(object):
             self.SamPlot.children[1].close()
             self.init_from_sam(sam)
             self.SamPlot.children =[self.stab,self.tab]
-            self.tab.set_trait('selected_index',2)
+            self.tab.set_trait('selected_index',1)
         else:
             self.close_all_tabs()
             self.ds[0].close()
@@ -726,6 +748,8 @@ class SAMGUI(object):
             self.selected.append(np.ones(sam_subcluster.adata.shape[0]).astype('bool'))
             self.selected_cells.append(np.array(list(sam_subcluster.adata.obs_names)))
             self.active_labels.append(np.zeros(sam_subcluster.adata.shape[0]))
+            self.dd_opts.append([''])
+            self.gene_expressions.append(np.zeros(sam_subcluster.adata.shape[0]))
             self.marker_genes.append(np.array(list(sam_subcluster.adata.var_names))[np.argsort(-sam_subcluster.adata.var['weights'].get_values())])
             self.marker_genes_tt.append('Genes ranked by SAM weights.')
             self.ds.append(0)
@@ -733,12 +757,15 @@ class SAMGUI(object):
             execute=True
 
         elif np.all(selected):
+            sam.preprocess_data(**self.preprocess_args)
+
             self.out.clear_output()
             with self.out:
                 sam.run(**self.run_args)
             self.marker_genes[i] = np.array(list(sam.adata.var_names))[np.argsort(-sam.adata.var['weights'].get_values())]
             self.marker_genes_tt[i] = 'Genes ranked by SAM weights.'
             execute=True
+            self.current_sam=sam
 
         if execute:
             title = self.rs_box.children[0].children[1].value
@@ -985,6 +1012,20 @@ class SAMGUI(object):
             disabled=False)#layout={'width':'30%'})
         close.on_click(self.close_tab)
 
+        hotkeys = widgets.Button(
+            description = 'ReadMe',
+            tooltip = '''While hovering over the scatter plot, the following keyboard inputs are available:
+                - Left/Right arrows: Scroll through & display ranked genes
+                - Shift: Random Forest classifier marker gene identification of selected cells
+                - Enter: SAM weights-based approach to marker gene identification of selected cells
+                - x: Unselect all cells
+                - c: Select all cells
+                - v: Reset view
+                - a: Toggle expression spatial averaging ON/OFF
+
+                Also note that clicking a cell will select/unselect all cells sharing its label.''',
+            disabled=True)#layout={'width':'30%'})
+
         amslider = widgets.Button(
             description = 'Opacity',
             tooltip = 'Changes the opacities of unselected points in the current plot.',
@@ -1027,12 +1068,18 @@ class SAMGUI(object):
         )
         mslider.observe(self.change_msize,'value')
 
+        acc = widgets.Dropdown(
+                value="",
+                options=[""]
+                )
+        acc.observe(self.pick_cells_dd,'value')
+
         return widgets.VBox([
             widgets.HBox([cp,cpm]),
             widgets.HBox([dp,dpm]),
             widgets.HBox([cl,clm]),
             widgets.HBox([l,cslider]),
-            widgets.HBox([da,dam]),
+            widgets.HBox([da,dam,acc]),
             widgets.HBox([dv,dvm]),
             widgets.HBox([irm,ism,avg]),
             widgets.HBox([us,usa,res]),
@@ -1044,11 +1091,12 @@ class SAMGUI(object):
             widgets.HBox([lsf,sf]),
             widgets.HBox([amslider,aslider]),
             widgets.HBox([lmslider,mslider]),
-            close
+            widgets.HBox([close,hotkeys])
         ])
     def reset_view(self,event):
         self.create_plot(self.stab.selected_index,self.stab.get_title(self.stab.selected_index))
 
+
     def save_data(self,path):
         path=path.value
         if path != '':
@@ -1058,6 +1106,12 @@ class SAMGUI(object):
             elif path.split('.')[-1] == 'p':
                 s = self.sams[self.stab.selected_index]
                 s.save(path)
+            elif (path.split('.')[-1] == 'png' or path.split('.')[-1] == 'pdf'
+                  or path.split('.')[-1] == 'eps' or path.split('.')[-1] == 'jpg'):
+                if len(path.split('/'))>1:
+                    ut.create_folder('/'.join(path.split('/')[:-1]))
+                self.stab.children[self.stab.selected_index].write_image(path)
+
     def show_expression(self,text):
         if type(text) is not str:
             gene = text.value
@@ -1095,22 +1149,22 @@ class SAMGUI(object):
                             a = x.flatten()
 
                         norm = self.preprocess_args.get('norm','log')
-                        norm = 'log'
+                        if norm is not None:
+                            if(norm.lower() == 'log'):
+                                a = np.log2(a + 1)
+                            elif(norm.lower() == 'ftt'):
+                                a = np.sqrt(a) + np.sqrt(a+1) - 1
+                            elif(norm.lower() == 'asin'):
+                                a = np.arcsinh(a)
+                else:
+                    norm = self.preprocess_args.get('norm','log')
+                    if norm is not None:
                         if(norm.lower() == 'log'):
                             a = np.log2(a + 1)
                         elif(norm.lower() == 'ftt'):
                             a = np.sqrt(a) + np.sqrt(a+1) - 1
                         elif(norm.lower() == 'asin'):
                             a = np.arcsinh(a)
-                else:
-                    norm = self.preprocess_args.get('norm','log')
-                    norm = 'log'
-                    if(norm.lower() == 'log'):
-                        a = np.log2(a + 1)
-                    elif(norm.lower() == 'ftt'):
-                        a = np.sqrt(a) + np.sqrt(a+1) - 1
-                    elif(norm.lower() == 'asin'):
-                        a = np.arcsinh(a)
 
                 if self.GENE_KEY!='':
                     title = gene+'; ' +str(s.adata.var[self.GENE_KEY].T[gene])
@@ -1119,7 +1173,7 @@ class SAMGUI(object):
 
                 self.select_all(None)
                 self.update_colors_expr(a,title)
-                self.gene_expression = a
+                self.gene_expressions[self.stab.selected_index] = a
             except IndexError:
                 0; # do nothing
 
@@ -1230,9 +1284,13 @@ class SAMGUI(object):
         self.cs_box.children[1].children[1].options = list(s.adata.obsm.keys())
         self.cs_box.children[4].children[1].options = [''] + list(s.adata.obs.keys())
         self.cs_box.children[5].children[1].options = [''] + list(s.adata.var.keys())
+        self.cs_box.children[4].children[2].options = self.dd_opts[i]
+
     def on_switch_tabs(self,event):
         self.update_dropdowns(self.stab.selected_index)
         self.cs_box.children[11].children[0].set_trait('tooltip',self.marker_genes_tt[self.stab.selected_index])
+        self.current_tab = self.stab.selected_index
+        self.current_sam = self.sams[self.current_tab]
 
 
     def display_var_annotation(self, event):
@@ -1242,6 +1300,7 @@ class SAMGUI(object):
         key = self.cs_box.children[4].children[1].value
         if key != '':
             labels = np.array(list(self.sams[self.stab.selected_index].adata.obs[key].get_values()))
+
             self.active_labels[self.stab.selected_index] = labels
             self.update_colors_anno(labels)
 
@@ -1258,13 +1317,20 @@ class SAMGUI(object):
 
 
         slider = self.cs_box.children[12].children[1]
-        slider.set_trait('max',a.max()+(a.max()-a.min())/100)
         slider.set_trait('min',0)
+        slider.set_trait('max',a.max()+(a.max()-a.min())/100)
         slider.set_trait('step',(a.max()-a.min())/100)
         slider.set_trait('value',0)
 
     def update_colors_anno(self,labels):
         nlabels = np.unique(labels).size
+        title = self.cs_box.children[4].children[1].value.split('_clusters')[0]
+
+        if issubclass(labels.dtype.type,np.number):
+            if issubclass(labels.dtype.type,np.float) or nlabels > 300:
+                self.update_colors_expr(labels,title)
+                return;
+
         if nlabels == 1:
             x = 'spectral'
         elif nlabels <= 2:
@@ -1279,15 +1345,18 @@ class SAMGUI(object):
 
         lbls,inv = np.unique(labels,return_inverse=True)
 
-        if type(labels.flatten()[0]) is str or type(labels.flatten()[0]) is np.str_:
+        dd = self.cs_box.children[4].children[2]
+        self.dd_opts[self.stab.selected_index] = [""] + list(lbls)
+        dd.options = self.dd_opts[self.stab.selected_index]
+
+        if issubclass(labels.dtype.type,np.character):
             tickvals=np.arange(lbls.size)
             ticktext=list(lbls)
         else:
             idx = np.round(np.linspace(0, len(lbls) - 1, 6)).astype(int)
             tickvals=list(idx)
-            ticktext=tickvals
+            ticktext=lbls[tickvals]
 
-        title = self.cs_box.children[4].children[1].value.split('_clusters')[0]
 
         f1 = self.stab.children[self.stab.selected_index]
         f1.update_traces(marker = dict(color = inv,colorscale=x,
@@ -1303,16 +1372,16 @@ class SAMGUI(object):
 
 
         slider = self.cs_box.children[12].children[1]
-        slider.set_trait('min',0)
-        slider.set_trait('max',0)
-        slider.set_trait('value',0)
+        slider.set_trait('min',slider.value)
+        slider.set_trait('max',slider.value)
+
 
     def threshold_selection(self, val):
         val = val['new']
         s = self.sams[self.stab.selected_index]
 
         self.selected[self.stab.selected_index][:]=False
-        self.selected[self.stab.selected_index][self.gene_expression>=val]=True
+        self.selected[self.stab.selected_index][self.gene_expressions[self.stab.selected_index]>=val]=True
         self.selected_cells[self.stab.selected_index]=np.array(list(s.adata.obs_names))[self.selected[self.stab.selected_index]]
         self.stab.children[self.stab.selected_index].data[0].selectedpoints = np.where(self.selected[self.stab.selected_index])[0]
 
@@ -1403,6 +1472,29 @@ class SAMGUI(object):
                                                         self.selected[self.stab.selected_index]]))
         trace.selectedpoints = list(np.where(self.selected[self.stab.selected_index])[0])
         trace.unselected = {'marker':{'opacity':self.cs_box.children[14].children[1].value}}
+
+    def pick_cells_dd(self,txt):
+        if txt['new'] != '':
+            al = txt['new']
+
+            sel = self.selected[self.stab.selected_index]
+            als = self.active_labels[self.stab.selected_index]
+            ratio = sel[als==al].sum() / sel[als==al].size
+            if ratio >= 0.5:
+                sel[als==al]=False
+            else:
+                sel[als==al]=True
+
+            self.selected_cells[self.stab.selected_index] = np.array(list(self.sams[
+                                                    self.stab.selected_index
+                                                    ].adata.obs_names[sel]))
+
+            self.stab.children[self.stab.selected_index].data[0].selectedpoints = list(np.where(sel)[0])
+
+            self.cs_box.children[4].children[2].value=''
+
+
+
     def pick_cells(self,trace, points, selector):
         tf = self.selected[self.stab.selected_index][points.point_inds[0]]
         als = self.active_labels[self.stab.selected_index]
